==> pwnkit-0.2.7.tar.gz: pwnkit-0.2.7/tests/test_encrypt.py <==
import pytest
from pwnkit.encrypt import PointerGuard, SafeLinking

def test_pointer_guard():
    print("[*] Testing PointerGuard...")
    g = 0xdeadbeefcafebabe
    pg = PointerGuard(guard=g)
    print(f"    guard       : {pg.guard:#018x}")

    ptr = 0x4141414142424242
    print(f"    pointer     : {ptr:#018x}")

    enc = pg.mangle(ptr)
    print(f"    encrypted   : {enc:#018x}")

    dec = pg.demangle(enc)
    print(f"    decrypted   : {dec:#018x}")

    # round-trip must recover the original pointer
    assert dec == ptr


def test_safelinking_encrypt_property():
    """
    Safe-linking is XOR-based: enc = fd ^ (heap_base >> 12).
    Validate this algebraic property directly.
    """
    print("[*] Testing SafeLinking encrypt property...")
    heap_base = 0x555555000000
    s = SafeLinking(heap_base=heap_base)

    fd = 0xdeadbeefcafebabe
    enc = s.encrypt(fd)
    print(f"    heap base   : {heap_base:#018x}")
    print(f"    fd          : {fd:#018x}")
    print(f"    encrypted   : {enc:#018x}")

    # Property: enc ^ fd == key
    key = heap_base >> 12
    assert (enc ^ fd) == key


def test_safelinking_decrypt_progressive_smoke():
    """
    Current decrypt_progressive() is a heuristic, not exact.
    We only sanity-check that the *very high* byte matches the original,
    which holds for the present implementation.
    """
    print("[*] Testing SafeLinking progressive (smoke)...")
    heap_base = 0x555555000000
    s = SafeLinking(heap_base=heap_base)

    fd = 0xdeadbeefcafebabe
    enc = s.encrypt(fd)
    dec = s.decrypt_progressive(enc)

    print(f"    heap base   : {heap_base:#018x}")
    print(f"    fd          : {fd:#018x}")
    print(f"    encrypted   : {enc:#018x}")
    print(f"    progressive : {dec:#018x}")

    # Very loose check: top byte matches (helps avoid false negatives
    # with the current progressive heuristic)
    assert (dec >> 56) == (fd >> 56)

==> pwnkit-0.2.8.tar.gz: pwnkit-0.2.8/tests/test_encrypt.py <==
import pytest
from pwnkit.encrypt import PointerGuard, SafeLinking

def test_pointer_guard():
    print("[*] Testing PointerGuard...")
    g = 0xdeadbeefcafebabe
    pg = PointerGuard(guard=g)
    print(f"    guard       : {pg.guard:#018x}")

    ptr = 0x4141414142424242
    print(f"    pointer     : {ptr:#018x}")

    enc = pg.mangle(ptr)
    print(f"    encrypted   : {enc:#018x}")

    dec = pg.demangle(enc)
    print(f"    decrypted   : {dec:#018x}")

    # round-trip must recover the original pointer
    assert dec == ptr


def test_safelinking_encrypt_property():
    """
    Safe-linking is XOR-based: enc = fd ^ (heap_base >> 12).
    Validate this algebraic property directly.
    """
    print("[*] Testing SafeLinking encrypt property...")
    heap_base = 0x555555000000
    s = SafeLinking(heap_base=heap_base)

    fd = 0xdeadbeefcafebabe
    enc = s.encrypt(fd)
    print(f"    heap base   : {heap_base:#018x}")
    print(f"    fd          : {fd:#018x}")
    print(f"    encrypted   : {enc:#018x}")

    # Property: enc ^ fd == key
    key = heap_base >> 12
    assert (enc ^ fd) == key


def test_safelinking_decrypt_smoke():
    """
    Current decrypt() is a heuristic, not exact.
    We only sanity-check that the *very high* byte matches the original,
    which holds for the present implementation.
    """
    print("[*] Testing SafeLinking (smoke)...")
    heap_base = 0x555555000000
    s = SafeLinking(heap_base=heap_base)

    fd = 0xdeadbeefcafebabe
    enc = s.encrypt(fd)
    dec = s.decrypt(enc)

    print(f"    heap base : {heap_base:#018x}")
    print(f"    fd        : {fd:#018x}")
    print(f"    encrypted : {enc:#018x}")
    print(f"    decrypted : {dec:#018x}")

    # Very loose check: top byte matches (helps avoid false negatives
    # with the current progressive heuristic)
    assert (dec >> 56) == (fd >> 56)
